import datetime
import logging

from django.conf import settings
from django.contrib.auth import get_user_model

from .signals import account_automatic_renewal

User = get_user_model()
logger = logging.getLogger('plans.tasks')


def get_active_plans():
    return User.objects.select_related('userplan').filter(userplan__active=True).exclude(userplan__expire=None)


def autorenew_account(providers=None):
    logger.info('Started automatic account renewal')
    PLANS_AUTORENEW_BEFORE_DAYS = getattr(settings, 'PLANS_AUTORENEW_BEFORE_DAYS', 0)
    PLANS_AUTORENEW_BEFORE_HOURS = getattr(settings, 'PLANS_AUTORENEW_BEFORE_HOURS', 0)

    accounts_for_renewal = get_active_plans().filter(
        userplan__recurring__has_automatic_renewal=True,
        userplan__recurring__token_verified=True,
        userplan__expire__lt=datetime.date.today() + datetime.timedelta(days=PLANS_AUTORENEW_BEFORE_DAYS,
                                                                        hours=PLANS_AUTORENEW_BEFORE_HOURS),
    )

    if providers:
        accounts_for_renewal = accounts_for_renewal.filter(userplan__recurring__payment_provider__in=providers)

    logger.info(f"{len(accounts_for_renewal)} accounts to be renewed.")

    for user in accounts_for_renewal.all():
        account_automatic_renewal.send(sender=None, user=user)
    return accounts_for_renewal


def expire_account():

    logger.info('Started account expiration')

    expired_accounts = get_active_plans().filter(userplan__expire__lt=datetime.date.today())

    for user in expired_accounts.all():
        user.userplan.expire_account()

    notifications_days_before = getattr(settings, 'PLANS_EXPIRATION_REMIND', [])

    if notifications_days_before:
        days = map(lambda x: datetime.date.today() + datetime.timedelta(days=x), notifications_days_before)
        for user in User.objects.select_related('userplan').filter(userplan__active=True, userplan__expire__in=days):
            user.userplan.remind_expire_soon()



from modeltranslation.translator import TranslationOptions, translator

from plans.base.models import AbstractPlan, AbstractPricing, AbstractQuota

Plan = AbstractPlan.get_concrete_model()
Pricing = AbstractPricing.get_concrete_model()
Quota = AbstractQuota.get_concrete_model()

# Translations for django-plans


class PlanTranslationOptions(TranslationOptions):
    fields = ('name', 'description', )


translator.register(Plan, PlanTranslationOptions)


class PricingTranslationOptions(TranslationOptions):
    fields = ('name',)


translator.register(Pricing, PricingTranslationOptions)


class QuotaTranslationOptions(TranslationOptions):
    fields = ('name', 'description', 'unit')


translator.register(Quota, QuotaTranslationOptions)



# Generated by Django 3.2.11 on 2022-01-19 17:14

from decimal import Decimal

import django.db.models.deletion
import django_countries.fields
import example.sample_plans.models
from django.conf import settings
from django.db import migrations, models

import plans.base.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Plan',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'order',
                    models.PositiveIntegerField(
                        db_index=True, editable=False, verbose_name='order'
                    ),
                ),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                (
                    'description',
                    models.TextField(blank=True, verbose_name='description'),
                ),
                (
                    'available',
                    models.BooleanField(
                        db_index=True,
                        default=False,
                        help_text='Is still available for purchase',
                        verbose_name='available',
                    ),
                ),
                (
                    'visible',
                    models.BooleanField(
                        db_index=True,
                        default=True,
                        help_text='Is visible in current offer',
                        verbose_name='visible',
                    ),
                ),
                (
                    'created',
                    models.DateTimeField(db_index=True, verbose_name='created'),
                ),
                (
                    'url',
                    models.URLField(
                        blank=True,
                        help_text='Optional link to page with more information (for clickable pricing table headers)',
                    ),
                ),
                (
                    'default',
                    models.BooleanField(
                        blank=True,
                        default=None,
                        help_text='Both "Unknown" and "No" means that the plan is not default',
                        null=True,
                    ),
                ),
                (
                    'customized',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name='customized',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Plan',
                'verbose_name_plural': 'Plans',
                'ordering': ('order',),
                'abstract': False,
            },
            bases=(plans.base.models.BaseMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Pricing',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                (
                    'period',
                    models.PositiveIntegerField(
                        blank=True,
                        db_index=True,
                        default=30,
                        null=True,
                        verbose_name='period',
                    ),
                ),
                (
                    'url',
                    models.URLField(
                        blank=True,
                        help_text='Optional link to page with more information (for clickable pricing table headers)',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Pricing',
                'verbose_name_plural': 'Pricings',
                'ordering': ('period',),
                'abstract': False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name='Quota',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'order',
                    models.PositiveIntegerField(
                        db_index=True, editable=False, verbose_name='order'
                    ),
                ),
                (
                    'codename',
                    models.CharField(
                        db_index=True,
                        max_length=50,
                        unique=True,
                        verbose_name='codename',
                    ),
                ),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                (
                    'unit',
                    models.CharField(blank=True, max_length=100, verbose_name='unit'),
                ),
                (
                    'description',
                    models.TextField(blank=True, verbose_name='description'),
                ),
                (
                    'is_boolean',
                    models.BooleanField(default=False, verbose_name='is boolean'),
                ),
                (
                    'url',
                    models.CharField(
                        blank=True,
                        help_text='Optional link to page with more information (for clickable pricing table headers)',
                        max_length=200,
                    ),
                ),
            ],
            options={
                'verbose_name': 'Quota',
                'verbose_name_plural': 'Quotas',
                'ordering': ('order',),
                'abstract': False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name='TestApp',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
            bases=(example.sample_plans.models.DetailFieldMixin, models.Model),
        ),
        migrations.CreateModel(
            name='UserPlan',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'expire',
                    models.DateField(
                        blank=True,
                        db_index=True,
                        default=None,
                        null=True,
                        verbose_name='expire',
                    ),
                ),
                (
                    'active',
                    models.BooleanField(
                        db_index=True, default=True, verbose_name='active'
                    ),
                ),
                (
                    'plan',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='sample_plans.plan',
                        verbose_name='plan',
                    ),
                ),
                (
                    'user',
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name='user',
                    ),
                ),
            ],
            options={
                'verbose_name': 'User plan',
                'verbose_name_plural': 'Users plans',
                'abstract': False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name='RecurringUserPlan',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'token',
                    models.CharField(
                        blank=True,
                        default=None,
                        help_text='Token, that will be used for payment renewal. Depends on used payment provider',
                        max_length=255,
                        null=True,
                        verbose_name='recurring token',
                    ),
                ),
                (
                    'payment_provider',
                    models.CharField(
                        blank=True,
                        default=None,
                        help_text='Provider, that will be used for payment renewal',
                        max_length=255,
                        null=True,
                        verbose_name='payment provider',
                    ),
                ),
                (
                    'amount',
                    models.DecimalField(
                        blank=True,
                        db_index=True,
                        decimal_places=2,
                        max_digits=7,
                        null=True,
                        verbose_name='amount',
                    ),
                ),
                (
                    'tax',
                    models.DecimalField(
                        blank=True,
                        db_index=True,
                        decimal_places=2,
                        max_digits=4,
                        null=True,
                        verbose_name='tax',
                    ),
                ),
                ('currency', models.CharField(max_length=3, verbose_name='currency')),
                (
                    'has_automatic_renewal',
                    models.BooleanField(
                        default=False,
                        help_text='Automatic renewal is enabled for associated plan. If False, the plan renewal can be still initiated by user.',
                        verbose_name='has automatic plan renewal',
                    ),
                ),
                (
                    'token_verified',
                    models.BooleanField(
                        default=False,
                        help_text='The recurring token has been verified by at least one payment to be working.',
                        verbose_name='token has been verified by payment',
                    ),
                ),
                ('card_expire_year', models.IntegerField(blank=True, null=True)),
                ('card_expire_month', models.IntegerField(blank=True, null=True)),
                (
                    'card_masked_number',
                    models.CharField(blank=True, max_length=255, null=True),
                ),
                (
                    'pricing',
                    models.ForeignKey(
                        blank=True,
                        default=None,
                        help_text='Recurring pricing',
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to='sample_plans.pricing',
                    ),
                ),
                (
                    'user_plan',
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='recurring',
                        to='sample_plans.userplan',
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name='PlanQuota',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                ('value', models.IntegerField(blank=True, default=1, null=True)),
                (
                    'plan',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='sample_plans.plan',
                    ),
                ),
                (
                    'quota',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='sample_plans.quota',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Plan quota',
                'verbose_name_plural': 'Plans quotas',
                'abstract': False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name='PlanPricing',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'price',
                    models.DecimalField(db_index=True, decimal_places=2, max_digits=7),
                ),
                ('order', models.IntegerField(default=0)),
                (
                    'has_automatic_renewal',
                    models.BooleanField(
                        default=False,
                        help_text='Use automatic renewal if possible?',
                        verbose_name='has automatic renewal',
                    ),
                ),
                (
                    'plan',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='sample_plans.plan',
                    ),
                ),
                (
                    'pricing',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='sample_plans.pricing',
                    ),
                ),
                (
                    'visible',
                    models.BooleanField(
                        default=True,
                        help_text='Is visible in current offer',
                        verbose_name='is visible by default',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Plan pricing',
                'verbose_name_plural': 'Plans pricings',
                'ordering': ('order', 'pricing__period'),
                'abstract': False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.AddField(
            model_name='plan',
            name='quotas',
            field=models.ManyToManyField(
                through='sample_plans.PlanQuota', to='sample_plans.Quota'
            ),
        ),
        migrations.CreateModel(
            name='Order',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                ('flat_name', models.CharField(blank=True, max_length=200, null=True)),
                (
                    'created',
                    models.DateTimeField(db_index=True, verbose_name='created'),
                ),
                (
                    'completed',
                    models.DateTimeField(
                        blank=True, db_index=True, null=True, verbose_name='completed'
                    ),
                ),
                (
                    'plan_extended_from',
                    models.DateField(
                        blank=True,
                        help_text='The plan was extended from this date',
                        null=True,
                        verbose_name='plan extended from',
                    ),
                ),
                (
                    'plan_extended_until',
                    models.DateField(
                        blank=True,
                        help_text='The plan was extended until this date',
                        null=True,
                        verbose_name='plan extended until',
                    ),
                ),
                (
                    'amount',
                    models.DecimalField(
                        db_index=True,
                        decimal_places=2,
                        max_digits=7,
                        verbose_name='amount',
                    ),
                ),
                (
                    'tax',
                    models.DecimalField(
                        blank=True,
                        db_index=True,
                        decimal_places=2,
                        max_digits=4,
                        null=True,
                        verbose_name='tax',
                    ),
                ),
                (
                    'currency',
                    models.CharField(
                        default='EUR', max_length=3, verbose_name='currency'
                    ),
                ),
                (
                    'status',
                    models.IntegerField(
                        choices=[
                            (1, 'new'),
                            (2, 'completed'),
                            (3, 'not valid'),
                            (4, 'canceled'),
                            (5, 'returned'),
                        ],
                        default=1,
                        verbose_name='status',
                    ),
                ),
                (
                    'plan',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='plan_order',
                        to='sample_plans.plan',
                        verbose_name='plan',
                    ),
                ),
                (
                    'pricing',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to='sample_plans.pricing',
                        verbose_name='pricing',
                    ),
                ),
                (
                    'user',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name='user',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Order',
                'verbose_name_plural': 'Orders',
                'ordering': ('-created',),
                'abstract': False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name='Invoice',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                ('number', models.IntegerField(db_index=True)),
                ('full_number', models.CharField(max_length=200)),
                (
                    'type',
                    models.IntegerField(
                        choices=[
                            (1, 'Invoice'),
                            (2, 'Invoice Duplicate'),
                            (3, 'Order confirmation'),
                        ],
                        db_index=True,
                        default=1,
                    ),
                ),
                ('issued', models.DateField(db_index=True)),
                (
                    'issued_duplicate',
                    models.DateField(blank=True, db_index=True, null=True),
                ),
                (
                    'selling_date',
                    models.DateField(blank=True, db_index=True, null=True),
                ),
                ('payment_date', models.DateField(db_index=True)),
                ('unit_price_net', models.DecimalField(decimal_places=2, max_digits=7)),
                ('quantity', models.IntegerField(default=1)),
                ('total_net', models.DecimalField(decimal_places=2, max_digits=7)),
                ('total', models.DecimalField(decimal_places=2, max_digits=7)),
                ('tax_total', models.DecimalField(decimal_places=2, max_digits=7)),
                (
                    'tax',
                    models.DecimalField(
                        blank=True,
                        db_index=True,
                        decimal_places=2,
                        max_digits=4,
                        null=True,
                    ),
                ),
                (
                    'rebate',
                    models.DecimalField(
                        decimal_places=2, default=Decimal('0'), max_digits=4
                    ),
                ),
                ('currency', models.CharField(default='EUR', max_length=3)),
                ('item_description', models.CharField(max_length=200)),
                ('buyer_name', models.CharField(max_length=200, verbose_name='Name')),
                (
                    'buyer_street',
                    models.CharField(max_length=200, verbose_name='Street'),
                ),
                (
                    'buyer_zipcode',
                    models.CharField(max_length=200, verbose_name='Zip code'),
                ),
                ('buyer_city', models.CharField(max_length=200, verbose_name='City')),
                (
                    'buyer_country',
                    django_countries.fields.CountryField(
                        default='PL', max_length=2, verbose_name='Country'
                    ),
                ),
                (
                    'buyer_tax_number',
                    models.CharField(
                        blank=True, max_length=200, verbose_name='TAX/VAT number'
                    ),
                ),
                (
                    'shipping_name',
                    models.CharField(max_length=200, verbose_name='Name'),
                ),
                (
                    'shipping_street',
                    models.CharField(max_length=200, verbose_name='Street'),
                ),
                (
                    'shipping_zipcode',
                    models.CharField(max_length=200, verbose_name='Zip code'),
                ),
                (
                    'shipping_city',
                    models.CharField(max_length=200, verbose_name='City'),
                ),
                (
                    'shipping_country',
                    django_countries.fields.CountryField(
                        default='PL', max_length=2, verbose_name='Country'
                    ),
                ),
                ('require_shipment', models.BooleanField(db_index=True, default=False)),
                ('issuer_name', models.CharField(max_length=200, verbose_name='Name')),
                (
                    'issuer_street',
                    models.CharField(max_length=200, verbose_name='Street'),
                ),
                (
                    'issuer_zipcode',
                    models.CharField(max_length=200, verbose_name='Zip code'),
                ),
                ('issuer_city', models.CharField(max_length=200, verbose_name='City')),
                (
                    'issuer_country',
                    django_countries.fields.CountryField(
                        default='PL', max_length=2, verbose_name='Country'
                    ),
                ),
                (
                    'issuer_tax_number',
                    models.CharField(
                        blank=True, max_length=200, verbose_name='TAX/VAT number'
                    ),
                ),
                (
                    'order',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='sample_plans.order',
                        verbose_name='order',
                    ),
                ),
                (
                    'user',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name='user',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Invoice',
                'verbose_name_plural': 'Invoices',
                'abstract': False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name='BillingInfo',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'tax_number',
                    models.CharField(
                        blank=True, db_index=True, max_length=200, verbose_name='VAT ID'
                    ),
                ),
                (
                    'name',
                    models.CharField(
                        db_index=True, max_length=200, verbose_name='name'
                    ),
                ),
                ('street', models.CharField(max_length=200, verbose_name='street')),
                ('zipcode', models.CharField(max_length=200, verbose_name='zip code')),
                ('city', models.CharField(max_length=200, verbose_name='city')),
                (
                    'country',
                    django_countries.fields.CountryField(
                        max_length=2, verbose_name='country'
                    ),
                ),
                (
                    'shipping_name',
                    models.CharField(
                        blank=True,
                        help_text='optional',
                        max_length=200,
                        verbose_name='name (shipping)',
                    ),
                ),
                (
                    'shipping_street',
                    models.CharField(
                        blank=True,
                        help_text='optional',
                        max_length=200,
                        verbose_name='street (shipping)',
                    ),
                ),
                (
                    'shipping_zipcode',
                    models.CharField(
                        blank=True,
                        help_text='optional',
                        max_length=200,
                        verbose_name='zip code (shipping)',
                    ),
                ),
                (
                    'shipping_city',
                    models.CharField(
                        blank=True,
                        help_text='optional',
                        max_length=200,
                        verbose_name='city (shipping)',
                    ),
                ),
                (
                    'user',
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name='user',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Billing info',
                'verbose_name_plural': 'Billing infos',
                'abstract': False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
    ]



from fabric.api import local, task


@task
def freeze_fixtures():
    local(
        'python manage.py dumpdata auth.User plans.BillingInfo plans.Invoice '
        'plans.Order plans.Plan plans.Pricing plans.PlanPricing plans.Quota '
        'plans.PlanQuota plans.UserPlan > example/foo/fixtures/initial_data.json',
    )



# Generated by Django 3.2.11 on 2022-01-13 19:17

import django_countries.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('plans', '0009_auto_20210303_1134'),
    ]

    operations = [
        migrations.AlterField(
            model_name='invoice',
            name='buyer_city',
            field=models.CharField(blank=True, max_length=200, verbose_name='City'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='buyer_country',
            field=django_countries.fields.CountryField(blank=True, default='PL', max_length=2, verbose_name='Country'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='buyer_name',
            field=models.CharField(blank=True, max_length=200, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='buyer_street',
            field=models.CharField(blank=True, max_length=200, verbose_name='Street'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='buyer_zipcode',
            field=models.CharField(blank=True, max_length=200, verbose_name='Zip code'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='shipping_city',
            field=models.CharField(blank=True, max_length=200, verbose_name='City'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='shipping_country',
            field=django_countries.fields.CountryField(blank=True, default='PL', max_length=2, verbose_name='Country'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='shipping_name',
            field=models.CharField(blank=True, max_length=200, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='shipping_street',
            field=models.CharField(blank=True, max_length=200, verbose_name='Street'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='shipping_zipcode',
            field=models.CharField(blank=True, max_length=200, verbose_name='Zip code'),
        ),
        migrations.AlterField(
            model_name='plan',
            name='default',
            field=models.BooleanField(db_index=True, default=None, help_text='Both "Unknown" and "No" means that the plan is not default', null=True, unique=True),
        ),
    ]



from django.core.management import BaseCommand

from plans import tasks


class Command(BaseCommand):
    help = 'Autorenew accounts and with recurring payments'

    def add_arguments(self, parser):
        parser.add_argument(
            '--providers',
            nargs='+',
            dest='providers',
            help='Renew only accounts with this providers',
        )

    def handle(self, *args, **options):  # pragma: no cover
        providers = options.get('providers')
        self.stdout.write("Starting renewal")
        renewed_accounts = tasks.autorenew_account(providers)
        if renewed_accounts:
            self.stdout.write("Accounts submitted to renewal:")
            for a in renewed_accounts:
                self.stdout.write(f'\t{a.userplan.recurring.payment_provider}\t{a.email}\t{a}')
        else:
            self.stdout.write("No accounts autorenewed")



import os
import sys
from decimal import Decimal

from django.conf import settings

sys.path[:0] = [os.path.join(os.getcwd(), 'demo')]


def pytest_configure(config):
    settings.configure(
        DATABASE_ENGINE='sqlite3',
        DATABASES={
            'default': {
                'NAME': ':memory:',
                'ENGINE': 'django.db.backends.sqlite3',
                'TEST_NAME': ':memory:',
            },
        },
        DATABASE_NAME=':memory:',
        TEST_DATABASE_NAME=':memory:',
        INSTALLED_APPS=[
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.admin',
            'django.contrib.sessions',
            'django.contrib.sites',
            'ordered_model',
            'example.foo',
            'plans',
        ],
        ROOT_URLCONF='example.urls',
        DEBUG=False,
        SITE_ID=1,
        TEMPLATE_DEBUG=True,
        USE_TZ=True,
        ALLOWED_HOSTS=['*'],
        PLANS_INVOICE_ISSUER={
            "issuer_name": "My Company Ltd",
            "issuer_street": "48th Suny street",
            "issuer_zipcode": "111-456",
            "issuer_city": "Django City",
            "issuer_country": "PL",
            "issuer_tax_number": "PL123456789",
        },
        PLANS_TAX=Decimal(23.0),
        PLANS_TAXATION_POLICY='plans.taxation.eu.EUTaxationPolicy',
        PLANS_TAX_COUNTRY='PL',
        PLANS_CURRENCY='PLN',
        PLANS_VALIDATORS={
            'MAX_FOO_COUNT': 'example.foo.validators.max_foos_validator',
        },
        EMAIL_BACKEND='django.core.mail.backends.console.EmailBackend',
    )



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


def get_country_code(request):
    if getattr(settings, 'PLANS_GET_COUNTRY_FROM_IP', False):
        try:
            from geolite2 import geolite2

            reader = geolite2.reader()
            ip_address = get_client_ip(request)
            ip_info = reader.get(ip_address)
        except ModuleNotFoundError:
            ip_info = None

        if ip_info and 'country' in ip_info:
            country_code = ip_info['country']['iso_code']
            return country_code
    return getattr(settings, 'PLANS_DEFAULT_COUNTRY', None)


def get_currency():
    CURRENCY = getattr(settings, 'PLANS_CURRENCY', '')
    if len(CURRENCY) != 3:
        raise ImproperlyConfigured('PLANS_CURRENCY should be configured as 3-letter currency code.')
    return CURRENCY


def country_code_transform(country_code):
    """ Transform country code to the code used by VIES """
    transform_dict = {
        "GR": "EL",
    }
    return transform_dict.get(country_code, country_code)



from django.contrib import admin
from modeltranslation.admin import TranslationAdmin

from plans.admin import PlanAdmin, QuotaAdmin
from plans.base.models import AbstractPlan, AbstractPricing, AbstractQuota

Plan = AbstractPlan.get_concrete_model()
Quota = AbstractQuota.get_concrete_model()
Pricing = AbstractPricing.get_concrete_model()

# Admin translation for django-plans


class TranslatedPlanAdmin(PlanAdmin, TranslationAdmin):
    pass


admin.site.unregister(Plan)
admin.site.register(Plan, TranslatedPlanAdmin)


class TranslatedPricingAdmin(TranslationAdmin):
    pass


admin.site.unregister(Pricing)
admin.site.register(Pricing, TranslatedPricingAdmin)


class TranslatedQuotaAdmin(QuotaAdmin, TranslationAdmin):
    pass


admin.site.unregister(Quota)
admin.site.register(Quota, TranslatedQuotaAdmin)



from plans.tests.tests import BillingInfoTestCase as BaseBillingInfoTestCase
from plans.tests.tests import \
    BillingInfoViewTestCase as BaseBillingInfoViewTestCase
from plans.tests.tests import \
    ConcurrentTestInvoice as BaseConcurrentTestInvoice
from plans.tests.tests import \
    CreateOrderViewTestCase as BaseCreateOrderViewTestCase
from plans.tests.tests import \
    EUTaxationPolicyTestCase as BaseEUTaxationPolicyTestCase
from plans.tests.tests import OrderTestCase as BaseOrderTestCase
from plans.tests.tests import \
    PlanChangePolicyTestCase as BasePlanChangePolicyTestCase
from plans.tests.tests import PlansTestCase as BasePlansTestCase
from plans.tests.tests import \
    RecurringPlansTestCase as BaseRecurringPlansTestCase
from plans.tests.tests import \
    StandardPlanChangePolicyTestCase as BaseStandardPlanChangePolicyTestCase
from plans.tests.tests import TasksTestCase as BaseTasksTestCase
from plans.tests.tests import TestInvoice as BaseTestInvoice
from plans.tests.tests import ValidatorsTestCase as BaseValidatorsTestCase


class PlanTestCase(BasePlansTestCase):
    pass


class TestInvoice(BaseTestInvoice):
    pass


class ConcurrentTestInvoice(BaseConcurrentTestInvoice):
    pass


class OrderTestCase(BaseOrderTestCase):
    pass


class PlanChangePolicyTestCase(BasePlanChangePolicyTestCase):
    pass


class StandardPlanChangePolicyTestCase(BaseStandardPlanChangePolicyTestCase):
    pass


class EUTaxationPolicyTestCase(BaseEUTaxationPolicyTestCase):
    pass


class BillingInfoTestCase(BaseBillingInfoTestCase):
    pass


class CreateOrderViewTestCase(BaseCreateOrderViewTestCase):
    pass


class ValidatorsTestCase(BaseValidatorsTestCase):
    pass


class BillingInfoViewTestCase(BaseBillingInfoViewTestCase):
    pass


class RecurringPlansTestCase(BaseRecurringPlansTestCase):
    pass


class TasksTestCase(BaseTasksTestCase):
    pass


del BaseBillingInfoTestCase
del BaseBillingInfoViewTestCase
del BaseConcurrentTestInvoice
del BaseCreateOrderViewTestCase
del BaseEUTaxationPolicyTestCase
del BaseOrderTestCase
del BasePlanChangePolicyTestCase
del BasePlansTestCase
del BaseRecurringPlansTestCase
del BaseStandardPlanChangePolicyTestCase
del BaseTasksTestCase
del BaseTestInvoice
del BaseValidatorsTestCase



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

# Django settings for example project.
import os
from decimal import Decimal

EMAIL_FROM = "Test <test@server.com>"

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SITE_ROOT = os.path.dirname(os.path.realpath(__file__))

DEBUG = True

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

ALLOWED_HOSTS = ['*']

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql',
        # Add 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': 'postgres',  # Or path to database file if using sqlite3.
        'USER': 'postgres',  # Not used with sqlite3.
        'PASSWORD': 'postgres',  # Not used with sqlite3.
        'HOST': 'localhost',  # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '5432',  # Set to empty string for default. Not used with sqlite3.
    }
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
    }
}

TIME_ZONE = 'America/Chicago'
USE_TZ = True
LANGUAGE_CODE = 'en'
SITE_ID = 1
USE_I18N = True
USE_L10N = True

LANGUAGES = (
    ('en', 'English'),
)

MEDIA_ROOT = ''
MEDIA_URL = ''
STATIC_ROOT = ''
STATIC_URL = '/static/'
ADMIN_MEDIA_PREFIX = '/static/admin/'
STATICFILES_DIRS = (
    os.path.join(SITE_ROOT, 'static'),
)

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    #    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

SECRET_KEY = 'l#^#iad$8$4=dlh74$!xs=3g4jb(&j+y6*ozy&8k1-&d+vruzy'

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(SITE_ROOT, 'templates'),
        ],
        'APP_DIRS': False,
        'OPTIONS': {
            'debug': True,
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.template.context_processors.request',
                'django.contrib.messages.context_processors.messages',
                'plans.context_processors.account_status',
            ],
            'loaders': [
                'django.template.loaders.filesystem.Loader',
                'django.template.loaders.app_directories.Loader',
            ]
        },
    },
]


ROOT_URLCONF = 'example.urls'
WSGI_APPLICATION = 'example.wsgi.application'

DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',

    'ordered_model',
    'bootstrap3',
    'django_concurrent_tests',

    'plans',
    'example.foo',
    'django_extensions',
    'sequences.apps.SequencesConfig',
]

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler'
        },

    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },

    }
}


# This is required for django-plans

PLANS_INVOICE_ISSUER = {
    "issuer_name": "My Company Ltd",
    "issuer_street": "48th Suny street",
    "issuer_zipcode": "111-456",
    "issuer_city": "Django City",
    "issuer_country": "PL",
    "issuer_tax_number": "PL123456789",
}

PLANS_TAX = Decimal('23.0')
PLANS_TAXATION_POLICY = 'plans.taxation.eu.EUTaxationPolicy'
PLANS_TAX_COUNTRY = 'PL'

PLANS_VALIDATORS = {
    'MAX_FOO_COUNT': 'example.foo.validators.max_foos_validator',
}

MANAGE_PY_PATH = os.environ.get("MANAGE_PY_PATH", './manage.py')

PLANS_CURRENCY = 'EUR'

EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

LOGIN_REDIRECT_URL = '/foo/list/'

ENABLE_FAKE_PAYMENTS = True

if os.environ.get('SAMPLE_APP', False):
    INSTALLED_APPS.remove('plans')
    INSTALLED_APPS.append('example.sample_plans')

    PLANS_PLAN_MODEL = 'sample_plans.Plan'
    PLANS_BILLINGINFO_MODEL = 'sample_plans.BillingInfo'
    PLANS_USERPLAN_MODEL = 'sample_plans.UserPlan'
    PLANS_PRICING_MODEL = 'sample_plans.Pricing'
    PLANS_PLANPRICING_MODEL = 'sample_plans.PlanPricing'
    PLANS_QUOTA_MODEL = 'sample_plans.Quota'
    PLANS_PLANQUOTA_MODEL = 'sample_plans.PlanQuota'
    PLANS_ORDER_MODEL = 'sample_plans.Order'
    PLANS_INVOICE_MODEL = 'sample_plans.Invoice'
    PLANS_RECURRINGUSERPLAN_MODEL = 'sample_plans.RecurringUserPlan'

    # Celery auto detects tasks only from INSTALLED_APPS
    CELERY_IMPORTS = ('openwisp_notifications.tasks',)



# Generated by Django 4.0.6 on 2022-07-28 13:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('plans', '0011_auto_20220208_1344'),
    ]

    operations = [
        migrations.AddField(
            model_name='planpricing',
            name='visible',
            field=models.BooleanField(default=True, help_text='Is visible in current offer', verbose_name='is visible by default'),
        ),
    ]



from django.apps import AppConfig

from . import conf as app_settings


class PlansConfig(AppConfig):
    name = 'plans'
    verbose_name = app_settings.APP_VERBOSE_NAME

    def ready(self):
        # noinspection PyUnresolvedReferences
        import plans.listeners  # noqa
